globalvar mapspin;
mapspin = id;

MAPSPIN_BASE_URL = "http://l3.ajf.me/mapspin/";
MAPSPIN_MAP_URL = "hhttp://l3.ajf.me/assets/maps/";
MAPSPIN_GENERATE = "generate/";
MAPSPIN_VOTE = "vote/"
MAPSPIN_LOGIN = "login2/";
MAPSPIN_REGISTER = "register2/";

ACTION_DL_ROTATION = 1;
ACTION_DL_MAP = 2;
ACTION_VOTE = 3;
ACTION_LOGIN = 4;
ACTION_REGISTER = 5;
ACTION_TEST = 6;

handle = -1;
downloadQueue = ds_queue_create();
actionQueue = ds_queue_create();
loggedIn = false;
queuedMaps = 0;
useRotation = false;
rotationTimeout = current_time;

ini_open("gg2.ini");
username = ini_read_string('Settings', 'Mapspin name', global.playerName);
key = ini_read_string('Settings', 'Mapspin key', '');
rotation = ini_read_string('Settings', 'Mapspin rotation', '');
tryLogin = ini_read_real('Settings', 'Mapspin trylogin', 1);

ini_write_string('Settings', 'Mapspin name', username);
ini_write_string('Settings', 'Mapspin key', key);
ini_write_string('Settings', 'Mapspin rotation', rotation);
ini_write_real('Settings', 'Mapspin trylogin', tryLogin);
ini_close();

// Create a non-blocking message system
execute_file(directory + "/" + "mapspinMessage.gml");

object_event_add(object_index, ev_step, ev_step_normal, "
    if (id != mapspin) exit;
    
    if (rotationTimeout + 1000*60*60 <= current_time && rotation != '') {
        ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_GENERATE + rotation);
        ds_queue_enqueue(actionQueue, ACTION_DL_ROTATION);
        rotationTimeout = current_time;
    }
    
    if (handle == -1) {
        if (!ds_queue_empty(downloadQueue))
            handle = http_new_get(ds_queue_dequeue(downloadQueue));
    } else if (http_step(handle)) {
        action = ds_queue_dequeue(actionQueue);
    
        // Request failed
        if (http_status_code(handle) != 200) {        
            var msg;
            msg = instance_create(0,0,MapspinMessage);
            msg.state = msg.STATE_MESSAGE;
            
            switch(action) {
                case ACTION_DL_ROTATION: msg.msg = 'Failed to download the maprotation#(l3.ajf.me is probably down)'; break;
                case ACTION_DL_MAP: msg.msg = 'Failed to download a map'; break;
                case ACTION_VOTE: msg.msg = 'Failed to vote'; break;
                case ACTION_LOGIN: msg.msg = 'Failed to login#(l3.ajf.me is probably down)'; break;
                case ACTION_REGISTER: msg.msg = 'Failed to register (l3.ajf.me is probably down)'; break;
                default: msg.msg = 'Unknown network error'; break;
            }

            http_destroy(handle);
            handle = -1;
            exit;
        }
        
        var data;
        data = http_response_body(handle);

        switch(action) {
            case ACTION_DL_ROTATION:
                rotationTimeout = current_time;
                
                //data = ggon_map_to_list(ggon_decode(read_string(data, buffer_size(data))));
                data = split(read_string(data, buffer_size(data)), ':');
                var list2, map;
                for(i=0; i<ds_list_size(data); i+=1) {
                    list2 = split(ds_list_find_value(data, i), '|');
                    map = ds_map_create();
                    ds_map_add(map, 'name', ds_list_find_value(list2, 0));
                    ds_map_add(map, 'hash', ds_list_find_value(list2, 1));
                    ds_map_add(map, 'url', ds_list_find_value(list2, 2));
                    ds_list_destroy(list2);
                    ds_list_replace(data, i, map);
                }
                
                if (ds_list_size(data) <= 0)
                    break;
                    
                ini_open('gg2.ini');
                ini_write_string('Settings', 'Mapspin rotation', rotation);
                ini_close();
                
                ds_list_clear(global.map_rotation); 
                for(i=0; i<ds_list_size(data); i+=1) {
                    mapData = ds_list_find_value(data, i);
                    if (CustomMapGetMapMD5(ds_map_find_value(mapData, 'name')) == ds_map_find_value(mapData, 'hash'))
                        ds_list_add(global.map_rotation, ds_map_find_value(mapData, 'name'));
                    else {
                        ds_queue_enqueue(downloadQueue, MAPSPIN_MAP_URL + ds_map_find_value(mapData, 'url'));
                        ds_queue_enqueue(actionQueue, ACTION_DL_MAP);
                        ds_queue_enqueue(actionQueue, ds_map_find_value(mapData, 'name'));
                        queuedMaps += 1;
                    }
                }
                
                for(i=0; i<ds_list_size(data); i+=1) {
                    ds_map_destroy(ds_list_find_value(data, i));
                }
                ds_list_destroy(data);
                
                if (queuedMaps <= 0)
                    event_user(0);
            break;
            case ACTION_DL_MAP:
                name = ds_queue_dequeue(actionQueue);
                write_buffer_to_file(data, 'Maps/' + name + '.png');
                ds_list_add(global.map_rotation, name);
                
                queuedMaps -= 1;
                if (queuedMaps <= 0)
                    event_user(0);
            break;
            case ACTION_VOTE:
                data = read_string(data, buffer_size(data));
                if (data != '') {
                    var msg;
                    msg = instance_create(0,0,MapspinMessage);
                    msg.state = msg.STATE_MESSAGE;
                    msg.msg = data;
                }
            break;
            case ACTION_LOGIN:
                data = read_string(data, buffer_size(data));
                if (data != '') {
                    var msg;
                    msg = instance_create(0,0,MapspinMessage);
                    msg.state = msg.STATE_LOGIN;
                    keyboard_string = username;
                } else {
                    loggedIn = true;
                    tryLogin = true;
                    
                    ini_open('gg2.ini');
                    ini_write_string('Settings', 'Mapspin name', username);
                    ini_write_string('Settings', 'Mapspin key', key);
                    ini_write_real('Settings', 'Mapspin trylogin', tryLogin);
                    ini_close();
                }
            break;
            case ACTION_REGISTER:
                data = read_string(data, buffer_size(data));
                if (data == '') {
                    var msg;
                    msg = instance_create(0,0,MapspinMessage);
                    msg.state = msg.STATE_MESSAGE;
                    msg.msg = 'Failed to register, please try again later';
                } else {
                    //data = ggon_map_to_list(ggon_decode(data));
                    data = split(data, ':');
                    
                    username = ds_list_find_value(data, 0);
                    key = ds_list_find_value(data, 1);
                    
                    ds_list_destroy(data);
                    
                    ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_LOGIN + '?username=' + username + '&key=' + key);
                    ds_queue_enqueue(actionQueue, ACTION_LOGIN);
                }
            break;
        }
        
        http_destroy(handle);
        handle = -1;
    }
");  

// Get maprotation
object_event_add(object_index, ev_alarm, 0, "
    if (id != mapspin) exit;
    
    if (tryLogin) {
        if (username != '' && key != '') {
            ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_LOGIN + '?username=' + username + '&key=' + key);
            ds_queue_enqueue(actionQueue, ACTION_LOGIN);
        } else {
            var msg;
            msg = instance_create(0,0,MapspinMessage);
            msg.state = msg.STATE_LOGIN;
            keyboard_string = username;
        }
    }

    if (global.isHost){
        var msg;
        msg = instance_create(0,0,MapspinMessage);
        msg.state = msg.STATE_ROTATION;
        keyboard_string = rotation;
            
        /*if (rotation != '') {
            ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_GENERATE + rotation + '/');
            ds_queue_enqueue(actionQueue, ACTION_DL_ROTATION);
        }*/
    }
");

// Vote
object_event_add(object_index, ev_keypress, vk_f3, "
    if (id != mapspin) exit;
    
    if ((!loggedIn && username != '' && key != '') || loggedIn) {
        if (!loggedIn) {
            ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_LOGIN + '?username=' + username + '&key=' + key);
            ds_queue_enqueue(actionQueue, ACTION_LOGIN);
        }  
        var msg;
        msg = instance_create(0,0,MapspinMessage);
        msg.state = msg.STATE_VOTE; 
    } else {
        var msg;
        msg = instance_create(0,0,MapspinMessage);
        msg.state = msg.STATE_LOGIN;
        keyboard_string = username;
    }
");

// Shuffle the new maprotation etc
object_event_add(object_index, ev_other, ev_user0, "
    queuedMaps = 0;
    GameServer.currentMapIndex = ds_list_size(global.map_rotation)-1;
    
    var map, i;
    if (global.shuffleRotation) {
        ds_list_shuffle(global.map_rotation);
        map = ds_list_find_value(global.map_rotation, 0);
        // 'Shuffle, don't make arena map first' chosen
        if (global.shuffleRotation == 1) {
            // if first map is arena
            if (string_copy(map, 0, 6) == 'arena_') {
                // try to find something else
                for (i = 0; i < ds_list_size(global.map_rotation); i += 1) {
                    map = ds_list_find_value(global.map_rotation, i);
                    // swap with first map
                    if (string_copy(map, 0, 6) != 'arena_') {
                        ds_list_replace(global.map_rotation, i, ds_list_find_value(global.map_rotation, 0));
                        ds_list_replace(global.map_rotation, 0, map);
                    }
                }
            }
        }
    }
    
    if (!useRotation) {
        useRotation = true;
        
        if (global.winners == -1)
            global.winners = TEAM_SPECTATOR;
            
        var message;
        message = 'Restarting maprotation because a new one has been downloaded.';
        ServerMessageString(message,global.sendBuffer);
        with NoticeO instance_destroy();
        notice = instance_create(0, 0, NoticeO);
        notice.notice = NOTICE_CUSTOM;
        notice.message = message;
    }
");    

// Login and start downloading the maprotation if this is a server
alarm[0] = 3;